import json
import os
import uuid
import datetime
import configparser

from PyQt5.QtWidgets import QWidget, QVBoxLayout, QLineEdit, QPushButton, QLabel, QFileDialog, QMessageBox

def create_tab():
    tab = QWidget()
    tab_layout = QVBoxLayout()

    # Add input fields and buttons
    name_label = QLabel("Enter co-worker's name (Name Surname):")
    name_input = QLineEdit()
    rct_label = QLabel("Enter reaction type and No. (e.g. CAT-001):")
    rct_input = QLineEdit()
    path_label = QLabel("Select folder to save JSON file:")
    path_input = QLineEdit()
    select_folder_button = QPushButton("Select Folder")
    create_button = QPushButton("Create JSON")

    tab_layout.addWidget(name_label)
    tab_layout.addWidget(name_input)
    tab_layout.addWidget(rct_label)
    tab_layout.addWidget(rct_input)
    tab_layout.addWidget(path_label)
    tab_layout.addWidget(path_input)
    tab_layout.addWidget(select_folder_button)
    tab_layout.addWidget(create_button)

    # Load config and set manufacturer if available
    config = configparser.ConfigParser()
    config.read('config.ini')
    metadata_template = config['MetadataTemplate']
    manufacturer = metadata_template.get('manufacturer', '')
    if manufacturer:
        name_input.setText(manufacturer)

    def select_folder():
        folder = QFileDialog.getExistingDirectory(None, "Select Folder")
        if folder:
            path_input.setText(folder)

    def create_json():
        name = name_input.text()
        rct_ident = rct_input.text()
        folder_path = path_input.text()

        # Use prefix and manufacturer from config if available, otherwise use name from GUI
        prefix = metadata_template.get('prefix', create_prefix(name))
        manufacturer = metadata_template.get('manufacturer', name)

        metadata = {
            "sample_id": prefix + "-" + rct_ident,
            "manufacturer": manufacturer,
            "uuid4": str(uuid.uuid4()),
            "sample_description": metadata_template.get('sample_description', ''),
            "sample_procedures": metadata_template.get('sample_procedures', ''),
            "parent": metadata_template.get('parent', ''),
            "child": metadata_template.get('child', ''),
            "analytics": {
                "XRD": metadata_template.get('analytics_XRD', '').split(', '),
                "Sorption": metadata_template.get('analytics_Sorption', '').split(', ')
            },
            "evaluation": {
                "XRD": metadata_template.get('evaluation_XRD', ''),
                "Sorption": metadata_template.get('evaluation_Sorption', '')
            },
            "dissemination": {
                "paper_01": metadata_template.get('dissemination_paper_01', '')
            }
        }

        if not folder_path:
            QMessageBox.warning(tab, "Warning", "Please select a folder to save the JSON file.")
            return

        create_json_file(folder_path, prefix + "-" + rct_ident + ".json", metadata)
        QMessageBox.information(tab, "Success", "JSON file created successfully.")

    select_folder_button.clicked.connect(select_folder)
    create_button.clicked.connect(create_json)

    tab.setLayout(tab_layout)
    return tab

def create_json_file(folder_path, filename, data):
    """Creates a JSON file with the current date in the filename."""

    if not os.path.exists(folder_path):
        os.makedirs(folder_path)

    # Create the current date in the format YYMMDD
    now = datetime.datetime.now()
    date_str = now.strftime("%y%m%d")

    # Prefix the filename with the date
    dated_filename = f"{date_str}_{filename}"

    file_path = os.path.join(folder_path, dated_filename)

    try:
        with open(file_path, 'w') as f:
            json.dump(data, f, indent=4, default=str)
        print(f"JSON file '{file_path}' created successfully.")
    except Exception as e:
        print(f"Error creating JSON file: {e}")

def create_prefix(name):
    """Creates a prefix from the first two letters of the first name and the first letter of the last name."""
    parts = name.split()  # Split the name into first and last name
    if len(parts) < 2:
        return "Invalid Name"  # Check if both first and last name are present

    first_name = parts[0]
    last_name = parts[1]

    prefix = first_name[:2] + last_name[0]  # Create the prefix
    return prefix.upper()  # Convert the prefix to uppercase